package com.mortals.xhx.thread;


import cn.hutool.core.util.IdUtil;
import com.alibaba.fastjson.JSON;
import com.mortals.framework.service.ICacheService;
import com.mortals.framework.util.AbstractThread;
import com.mortals.xhx.base.system.message.impl.MessageProducer;
import com.mortals.xhx.busiz.req.DeviceMsgReq;
import com.mortals.xhx.common.code.LogTypeEnum;
import com.mortals.xhx.common.key.Constant;
import com.mortals.xhx.common.key.QueueKey;
import com.mortals.xhx.common.model.DefaultTbQueueMsgHeaders;
import com.mortals.xhx.common.model.MessageHeader;
import com.mortals.xhx.module.device.model.DeviceEntity;
import com.mortals.xhx.module.device.model.DeviceLogEntity;
import com.mortals.xhx.module.device.service.DeviceLogService;
import com.mortals.xhx.module.device.service.DeviceService;
import com.mortals.xhx.queue.DefaultTbQueueMsg;
import com.mortals.xhx.queue.TbQueueMsg;
import com.mortals.xhx.queue.TbQueueMsgHeaders;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.core.annotation.Order;
import org.springframework.stereotype.Service;
import org.springframework.util.ObjectUtils;

import java.util.Date;

import static com.mortals.xhx.common.key.RedisKey.KEY_DEVICE_DOWN_MSG_QUEUE;


/**
 * 发送第三方服务线程
 *
 * @author zxfei
 */
@Slf4j
@Service
@Order(value = 3)
public class DeviceDownMsgThread extends AbstractThread {

    @Autowired
    private ICacheService cacheService;
    @Autowired
    private DeviceService deviceService;
    @Autowired
    private DeviceLogService deviceLogService;
    @Autowired
    private MessageProducer messageProducer;


    @Override
    protected int getSleepTime() {
        return 1000;
    }

    @Override
    protected void process() {
        log.info("DeviceDownMsgThread process");
        DeviceMsgReq deviceMsgReq = cacheService.blpop(KEY_DEVICE_DOWN_MSG_QUEUE, 10, DeviceMsgReq.class);
        if (!ObjectUtils.isEmpty(deviceMsgReq)) {
            log.info("下发消息:{}", deviceMsgReq.getDeviceCode());
            //根据设备编码查询设备
            DeviceEntity deviceEntity = null;
            try {
                deviceEntity = deviceService.getExtCache(deviceMsgReq.getDeviceCode());
            } catch (Exception e) {
                log.error("redis 获取设备异常！", e);
            }
            if (!ObjectUtils.isEmpty(deviceEntity)) {
                TbQueueMsgHeaders header = new DefaultTbQueueMsgHeaders();
                header.put(MessageHeader.MESSAGETYPE, deviceMsgReq.getMessageType());
                header.put(MessageHeader.DEVICECODE, deviceMsgReq.getDeviceCode());
                header.put(MessageHeader.TIMESTAMP, deviceMsgReq.getTimestamp().toString());

                TbQueueMsg queueMsg = new DefaultTbQueueMsg(IdUtil.fastUUID(), deviceMsgReq.getData(), header);
                messageProducer.sendMsg(QueueKey.DEFAULT_EXCHANGE, Constant.DOWN_TOPIC + deviceEntity.getDeviceCode(), JSON.toJSONString(queueMsg));

                DeviceLogEntity deviceLogEntity = new DeviceLogEntity();
                deviceLogEntity.initAttrValue();
                deviceLogEntity.setTraceID(IdUtil.fastSimpleUUID());
                deviceLogEntity.setSiteId(deviceEntity.getSiteId());
                deviceLogEntity.setDeviceId(deviceEntity.getId());
                deviceLogEntity.setDeviceName(deviceEntity.getDeviceName());
                deviceLogEntity.setDeviceCode(deviceEntity.getDeviceCode());
                deviceLogEntity.setMessageHead(deviceMsgReq.getMessageType());
                deviceLogEntity.setContent(deviceMsgReq.getData());
                deviceLogEntity.setLogType(LogTypeEnum.下发服务.getValue());
                deviceLogEntity.setCreateUserId(1L);
                deviceLogEntity.setCreateTime(new Date());
                deviceLogService.save(deviceLogEntity, null);
            } else {
                log.info("未找到设备，deviceCode:{}", deviceMsgReq.getDeviceCode());
            }
        }
    }


    @Override
    protected void threadClosed() {
        // TODO Auto-generated method stub
        super.close();
    }


}
